"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PLATFORM_NAME = exports.PLUGIN_NAME = void 0;
const motionMount_1 = require("../helper/motionMount");
exports.PLUGIN_NAME = 'homebridge-vogels-motionmount';
exports.PLATFORM_NAME = 'MotionMountDynamicPlatform';
let Accessory;
class MotionMountDynamicPlatform {
    constructor(log, config, api) {
        this.log = log;
        this.api = api;
        this.hap = api.hap;
        Accessory = api.platformAccessory;
        this.api.on("didFinishLaunching" /* APIEvent.DID_FINISH_LAUNCHING */, async () => {
            if (!this.tvAccessory) {
                this.tvAccessory = await this.createTvAccessory(config.name || 'MotionMount');
                this.api.registerPlatformAccessories(exports.PLUGIN_NAME, exports.PLATFORM_NAME, [
                    this.tvAccessory,
                ]);
            }
            await this.updateInputs();
        });
    }
    async configureAccessory(accessory) {
        this.log.info('ConfigureAccessory');
        const tvService = accessory.getService(this.hap.Service.Television);
        if (!tvService) {
            this.log.warn('[configureAccessory] Not a tv accessory');
            return;
        }
        this.tvAccessory = accessory;
        tvService
            .getCharacteristic(this.hap.Characteristic.Active)
            .on('set', (active, callback) => {
            if (active) {
                callback(null);
                return;
            }
            this.moveToWall(tvService, callback);
        });
        tvService
            .getCharacteristic(this.hap.Characteristic.ActiveIdentifier)
            .on('set', async (index, callback) => {
            await (0, motionMount_1.moveToPosition)(this.tvAccessory.context.positionPresets[index], this.log);
            callback(null);
        });
    }
    async moveToWall(tvService, callback) {
        await (0, motionMount_1.moveToPosition)(motionMount_1.WALL_POSITION, this.log);
        tvService.updateCharacteristic(this.hap.Characteristic.ActiveIdentifier, 0);
        callback(null);
    }
    async createTvAccessory(displayName) {
        // Accessory config
        const tvAccessory = new Accessory(displayName, this.hap.uuid.generate('homebridge:vogels-motionmount'));
        tvAccessory.category = 31 /* Categories.TELEVISION */;
        // TV service
        const tvService = tvAccessory.addService(this.hap.Service.Television);
        tvService.setCharacteristic(this.hap.Characteristic.ConfiguredName, displayName);
        tvService.setCharacteristic(this.hap.Characteristic.SleepDiscoveryMode, this.hap.Characteristic.SleepDiscoveryMode.ALWAYS_DISCOVERABLE);
        return tvAccessory;
    }
    arePositionPresetsUnchanged(positionPresets) {
        const lastKnownPositionPresets = this.tvAccessory.context.positionPresets || [];
        return (lastKnownPositionPresets.length === positionPresets.length &&
            !lastKnownPositionPresets.some(({ label, hexPosition }, index) => positionPresets[index].label !== label ||
                positionPresets[index].hexPosition !== hexPosition));
    }
    async updateInputs() {
        const tvService = this.tvAccessory.getService(this.hap.Service.Television);
        if (!tvService)
            throw new Error('Missing TV service');
        const positionPresets = await (0, motionMount_1.retrievePositionPresets)(this.log);
        if (this.arePositionPresetsUnchanged(positionPresets)) {
            this.log.info('[updateInputs] position presets unchanged');
            return positionPresets;
        }
        this.log.info('[updateInputs] Nuking all inputs');
        // Nuke presets
        this.tvAccessory.services
            .filter(({ displayName }) => !!displayName)
            .forEach((service) => this.tvAccessory.removeService(service));
        positionPresets.forEach(({ label }, index) => {
            this.log.info('[updateInputs] Adding input for preset', label);
            const inputService = this.tvAccessory.addService(this.hap.Service.InputSource, label.toLowerCase(), label);
            inputService
                .setCharacteristic(this.hap.Characteristic.Identifier, index)
                .setCharacteristic(this.hap.Characteristic.ConfiguredName, label)
                .setCharacteristic(this.hap.Characteristic.IsConfigured, this.hap.Characteristic.IsConfigured.CONFIGURED)
                .setCharacteristic(this.hap.Characteristic.InputSourceType, this.hap.Characteristic.InputSourceType.OTHER);
            tvService.addLinkedService(inputService);
        });
        this.tvAccessory.context.positionPresets = positionPresets;
        return positionPresets;
    }
}
exports.default = MotionMountDynamicPlatform;
